#need to return the following data with the corresponding units:
#time [s]
#shock radius [km]
#protoneutron star radius  [where \rho=1e11 g/cm^3, km]
#heating rate in the gain region [10^{51} erg/s]
#mass accretion rate [Msun/s; >0]
#Luminosity, lab frame, 500km, nue [10^{51} erg/s]
#Luminosity, lab frame, 500km, anti_nue [10^{51} erg/s]
#Luminosity, lab frame, 500km, nux = 1/4 {numu,antinumu,nutau,anti-nutau} [10^{51} erg/s]
#average energy, lab frame, 500km, nue [MeV]
#average energy, lab frame, 500km, anti_nue [MeV]
#average energy, lab frame, 500km, nux = 1/4 {numu,antinumu,nutau,anti-nutau} [MeV]
#rms energy, lab frame, 500km, nue [MeV]
#rms energy, lab frame, 500km, anti_nue [MeV]
#rms energy, lab frame, 500km, nux = 1/4 {numu,antinumu,nutau,anti-nutau} [MeV]
import numpy as np
from smooth import smooth


def get_GR1DData(directoryname):
    file = directoryname+"/tbounce.dat"
    tbounce = np.loadtxt(file)
    file = directoryname+"/shock_radius_t_corrected.dat"
    time, rsh  = np.loadtxt(file,usecols=(0,2), unpack=True) #second column is bugged shock radius, third is true location of min(v)
    rsh = rsh/1e5
    file = directoryname+"/accretion_rates.dat"
    time, massaccret  = np.loadtxt(file,usecols=(0,11), unpack=True)
    file = directoryname+"/M1_flux_lum.dat"
    time, lnue, lanue, lnux = np.loadtxt(file,usecols=(0,1,2,3), unpack=True)
    lnue = lnue/1.0e51
    lanue = lanue/1.0e51
    lnux = lnux/4.0e51
    file = directoryname+"/M1_flux_aveenergy_lab.dat"
    time, enue, eanue, enux = np.loadtxt(file,usecols=(0,1,2,3), unpack=True)
    file = directoryname+"/M1_flux_rmsenergy_lab.dat"
    time, rnue, ranue, rnux = np.loadtxt(file,usecols=(0,1,2,3), unpack=True)
    file = directoryname+"/r_rho1e11.dat"
    time, rpns = np.loadtxt(file,usecols=(0,1), unpack=True)
    rpns = rpns/1e5
    file = directoryname+"/M1_net_heating.dat"
    time, heating = np.loadtxt(file,usecols=(0,5), unpack=True) #use the deintdt column, not the denerdt column
    heating = heating/1.0e51
    time = time - tbounce    

    #shock radius file only starts at bounce, need to fill in eariler data so all time arrays agree
    shock_len = len(rsh)
    real_len = len(time)
    for i in range(real_len-shock_len):
        rsh = np.insert(rsh,0,0.0)

    win = 5
    time = smooth(time,window_len=win,window="flat")
    rsh = smooth(rsh,window_len=win,window="flat")
    heating = smooth(heating,window_len=win,window="flat")
    rpns = smooth(rpns,window_len=win,window="flat")
    massaccret = smooth(massaccret,window_len=win,window="flat")
    lnue = smooth(lnue,window_len=win,window="flat")
    lanue = smooth(lanue,window_len=win,window="flat")
    lnux = smooth(lnux,window_len=win,window="flat")
    enue = smooth(enue,window_len=win,window="flat")
    eanue = smooth(eanue,window_len=win,window="flat")
    enux = smooth(enux,window_len=win,window="flat")
    rnue = smooth(rnue,window_len=win,window="flat")
    ranue = smooth(ranue,window_len=win,window="flat")
    rnux = smooth(rnux,window_len=win,window="flat")

    for i in range(len(time)):
        if time[i] < -0.001:
            enux[i] = 0.0
            eanue[i] = 0.0

        if time[i] < 0.02:
            heating[i] = 0.0

    for i in range(len(enue)):
        enue[i] = max(0.01,enue[i])
        eanue[i] = max(0.01,eanue[i])
        enux[i] = max(0.01,enux[i])
        rnue[i] = max(0.01,rnue[i])
        ranue[i] = max(0.01,ranue[i])
        rnux[i] = max(0.01,rnux[i])


            
    return time,rsh,massaccret,lnue,lanue,lnux,enue,eanue,enux,rpns,heating,rnue,ranue,rnux


def get_FLASHData():
    file = "./FLASH/s20_18_finalJPG_small.dat"
    #note, no guarntee about what the other data columns are or if they are correct.
    time, rsh, massaccret, rpns, heating,lnue, lanue, lnux, enue, eanue, enux,rnue,ranue,rnux = np.loadtxt(file,usecols=(0,11,13,29,17,33,34,35,36,37,38,39,40,41), unpack=True)
    rsh = rsh/1e5
    rpns = rpns/1e5
    massaccret = massaccret/1.9889e33
    heating = heating/1e51
    lnux = lnux/4.0
    time = time + 0.01483

    for i in range(len(enue)):
        enue[i] = max(0.01,enue[i])
        eanue[i] = max(0.01,eanue[i])
        enux[i] = max(0.01,enux[i])
        rnue[i] = max(0.01,rnue[i])
        ranue[i] = max(0.01,ranue[i])
        rnux[i] = max(0.01,rnux[i])
    
    return time,rsh,massaccret,lnue,lanue,lnux,enue,eanue,enux,rpns,heating,rnue,ranue,rnux

def get_FornaxData():
    file = "./Fornax/nosmooth_anti_fix_20_SFHO_noINS_bruenn.new_brem1_nr16.dat"
    time, rsh, massaccret, rpns, lnue, lanue, lnux, enue, eanue, enux, rnue,ranue,rnux = np.loadtxt(file,usecols=(0,1,2,3,4,5,6,7,8,9,10,11,12), unpack=True)
    file = "./Fornax/20_SFHO_noINS_bruenn.new_brem1_nr16_Qdot.dat"
    time, heating = np.loadtxt(file,usecols=(0,1), unpack=True)
    rsh = rsh
    rpns = rpns/1e5
    lnue = 10.0*lnue
    lanue = 10.0*lanue
    lnux = 10.0*lnux

    for i in range(len(enue)):
        enue[i] = max(0.01,enue[i])
        eanue[i] = max(0.01,eanue[i])
        enux[i] = max(0.01,enux[i])
        rnue[i] = max(0.01,rnue[i])
        ranue[i] = max(0.01,ranue[i])
        rnux[i] = max(0.01,rnux[i])
        
    return time,rsh,massaccret,lnue,lanue,lnux,enue,eanue,enux,rpns,heating,rnue,ranue,rnux

def get_3DSNIDSAData():
    file = "./3DnSNe-IDSA/tprof.dat"
    time, rsh, massaccret, lnue, lanue, lnux, enue, eanue, enux,heating,rpns,rnue,ranue,rnux = np.loadtxt(file,usecols=(0,1,2,3,4,5,6,7,8,9,11,12,13,14), unpack=True)
    rsh = rsh/1e5
    lnue = lnue/1e51
    lanue = lanue/1e51
    lnux = lnux/1.0e51
    rpns = rpns/1e5
    heating = heating/1e51

    time = time - 0.002 #issue with original subtraction of the bounce time in the data
    
    for i in range(len(enue)):
        enue[i] = max(0.01,enue[i])
        eanue[i] = max(0.01,eanue[i])
        enux[i] = max(0.01,enux[i])
        rnue[i] = max(0.01,rnue[i])
        ranue[i] = max(0.01,ranue[i])
        rnux[i] = max(0.01,rnux[i])
    
    return time,rsh,massaccret,lnue,lanue,lnux,enue,eanue,enux,rpns,heating,rnue,ranue,rnux


def get_VertexData():
    file = "./Vertex/Quantities_s20.0_SFHo_with_escat_final.txt"
    time, rpns, rgain, rsh, heating, massaccret, lnue, lanue, lnux, enue, eanue, enux, e1e,e1a,e1x,r1e,r1a,r1x = np.loadtxt(file, unpack=True)
    massaccret = -massaccret

    time = time-0.2978
    
    for i in range(len(time)):
        if time[i] < 0.02:
            heating[i] = 0.0

    for i in range(len(e1e)):
        e1e[i] = max(0.01,e1e[i])
        e1a[i] = max(0.01,e1a[i])
        e1x[i] = max(0.01,e1x[i])
        r1e[i] = max(0.02,r1e[i])
        r1a[i] = max(0.02,r1a[i])
        r1x[i] = max(0.02,r1x[i])
            
    ae = (2.0*e1e*e1e-r1e**2)/(r1e**2-e1e**2)
    aa = (2.0*e1a*e1a-r1a**2)/(r1a**2-e1a**2)
    ax = (2.0*e1x*e1x-r1x**2)/(r1x**2-e1x**2)

    rnue = np.sqrt((2.0+ae)/(1.0+ae)*enue**2)
    ranue = np.sqrt((2.0+aa)/(1.0+aa)*eanue**2)
    rnux = np.sqrt((2.0+ax)/(1.0+ax)*enux**2)

    for i in range(len(enue)):
        enue[i] = max(0.01,enue[i])
        eanue[i] = max(0.01,eanue[i])
        enux[i] = max(0.01,enux[i])
        rnue[i] = max(0.01,rnue[i])
        ranue[i] = max(0.01,ranue[i])
        rnux[i] = max(0.01,rnux[i])

        
    return time,rsh,massaccret,lnue,lanue,lnux,enue,eanue,enux,rpns,heating,rnue,ranue,rnux

def get_ABData():

    tbounce = 0.4194116090912149

    time_01,rsh_01 = np.loadtxt("./AB/01_shock.dat",unpack=True,usecols=(0,1),skiprows=2)
    time_02,rpns_02  = np.loadtxt("./AB/02_PNS_radius.dat",unpack=True,skiprows=2)
    time_03,mdot_03  = np.loadtxt("./AB/03_mdot.dat",unpack=True,usecols=(1,2),skiprows=2)
    time_04,heating_04 = np.loadtxt("./AB/heating_from_rates.dat",unpack=True)
    time_05,lnue,lanue,lnux,lanux  = np.loadtxt("./AB/luminosity_rest.dat",unpack=True,skiprows=2)
    time_08,enue,eanue,enux,eanux  = np.loadtxt("./AB/menergy_rest.dat",unpack=True,skiprows=2)
    time_11,rnue,ranue,rnux,ranux  = np.loadtxt("./AB/rmsenergy_rest.dat",unpack=True,skiprows=2)

    #take time_05 to be the base, interpolate data to that basis since some are different
    time = time_05
    base_length = len(time_05)
    rsh = lnue*0.0
    rpns = lnue*0.0
    mdot = lnue*0.0
    heating = lnue*0.0

    tiny = 1.0e-10
    for i in range(base_length):
        target_time = time[i]
        #shock radius
        if target_time < time_01[0]:
            rsh[i] = 0.0
        else:
            j=1
            while target_time >= time_01[j]-tiny and j<len(time_01)-1:
                j = j+1

            #time between j-1 and j, interpolate
            rsh[i] = rsh_01[j-1] + (rsh_01[j]-rsh_01[j-1])/(time_01[j]-time_01[j-1])*(target_time-time_01[j-1])

        #PNS radius
        if target_time < time_02[0]:
            rpns[i] = 0.0
        else:
            j=1
            while target_time >= time_02[j]-tiny and j<len(time_02)-1:
                j = j+1

            #time between j-1 and j, interpolate
            rpns[i] = rpns_02[j-1] + (rpns_02[j]-rpns_02[j-1])/(time_02[j]-time_02[j-1])*(target_time-time_02[j-1])

        #Mass accretion rate
        if target_time < time_03[0]:
            mdot[i] = -1.0
        else:
            j=1
            while target_time >= time_03[j]-tiny and j<len(time_03)-1:
                j = j+1

            #time between j-1 and j, interpolate
            mdot[i] = mdot_03[j-1] + (mdot_03[j]-mdot_03[j-1])/(time_03[j]-time_03[j-1])*(target_time-time_03[j-1])

        #heating rate
        if target_time < time_04[0]:
            heating[i] = 0.0
        else:
            j=1
            while target_time >= time_04[j]-tiny and j<len(time_04)-1:
                j = j+1

            #time between j-1 and j, interpolate
            heating[i] = heating_04[j-1] + (heating_04[j]-heating_04[j-1])/(time_04[j]-time_04[j-1])*(target_time-time_04[j-1])            

    lnux = (lnux+lanux)*0.5
    enux = (enux+eanux)*0.5
    rnux = (rnux+ranux)*0.5

    #smooth lums and heating
    for i in  range(base_length):
        if i<2 or i+2 >= base_length:
            #leave it been
            lnue[i] = lnue[i]
            lanue[i] = lanue[i]
            lnux[i] = lnux[i]
            heating[i] = heating[i]
        else:
            lnue[i] = sum(lnue[i-2:i+3])/5.0
            lanue[i] = sum(lanue[i-2:i+3])/5.0                            
            lnux[i] = sum(lnux[i-2:i+3])/5.0                
            heating[i] = sum(heating[i-2:i+3])/5.0    
            
    mdot = -mdot
    lnue = lnue/1.0e51
    lanue = lanue/1.0e51
    lnux = lnux/1.0e51
    rpns = rpns/1.0e5
    rsh = rsh/1.0e5
    heating = heating/1.0e51
    
    return time,rsh,mdot,lnue,lanue,lnux,enue,eanue,enux,rpns,heating,rnue,ranue,rnux
